#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['b2luigi',
 'b2luigi.basf2_helper',
 'b2luigi.batch',
 'b2luigi.batch.processes',
 'b2luigi.cli',
 'b2luigi.core']

package_data = \
{'': ['*']}

install_requires = \
['luigi<2.7.9',
 'parse>=1.8.4',
 'GitPython>=2.1.11',
 'colorama>=0.3.9',
 'cachetools>=2.1.0']

setup(name='b2luigi',
      version='0.4.2',
      description='Task scheduling and batch running for basf2 jobs made simple',
      author='Nils Braun',
      author_email='nils.braun@kit.edu',
      url='https://github.com/nils-braun/b2luigi',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
